<?php

namespace Drupal\Tests\access_policy\Kernel\Plugin\AccessRule;

use Drupal\access_policy\Entity\AccessPolicy;
use Drupal\access_policy\Plugin\OperatorValidationTrait;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\Tests\access_policy\Kernel\AccessPolicyKernelTestBase;

/**
 * Tests EntityFieldListNumeric plugin features.
 *
 * @group access_policy
 */
class EntityFieldListNumericTest extends AccessPolicyKernelTestBase {

  use TaxonomyTestTrait;
  use OperatorValidationTrait;

  /**
   * The tags vocabulary.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $vocabulary;

  /**
   * The modules to install.
   *
   * @var string[]
   */
  protected static $modules = [
    'field',
    'filter',
    'node',
    'options',
    'access_policy',
    'access_policy_test',
    'system',
    'text',
    'user',
    'datetime',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['access_policy_test']);

    FieldStorageConfig::create([
      'field_name' => 'field_list_integer',
      'entity_type' => 'node',
      'type' => 'list_integer',
      'cardinality' => 1,
      'settings' => [
        'allowed_values' => [
          0 => 'Public',
          1 => 'Confidential',
          2 => 'Secret',
        ],
      ],
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_list_integer',
      'entity_type' => 'node',
      'label' => 'Security level',
      'bundle' => 'page',
    ])->save();

    FieldStorageConfig::create([
      'field_name' => 'field_list_integer',
      'entity_type' => 'user',
      'type' => 'list_integer',
      'cardinality' => 1,
      'settings' => [
        'allowed_values' => [
          0 => 'Public',
          1 => 'Confidential',
          2 => 'Secret',
        ],
      ],
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_list_integer',
      'entity_type' => 'user',
      'label' => 'Security clearance',
      'bundle' => 'user',
    ])->save();

    // Creating a policy is necessary to register the plugins.
    $policy = AccessPolicy::create([
      'id' => 'security',
      'label' => 'Security level',
      'target_entity_type_id' => 'node',
    ]);
    $policy->save();
  }

  /**
   * Tests numeric field lists with numeric operators.
   */
  public function testEntityFieldListNumeric() {

    $policy = $this->entityTypeManager->getStorage('access_policy')->load('security');
    $access_rule = $this->accessRuleManager->getHandler('node', 'numeric_field_list_integer');
    $access_rule->setSettings([
      'value' => '1',
      'operator' => '>',
      'query' => TRUE,
    ]);
    $policy->addAccessRule($access_rule);
    $policy->save();

    // Create a basic node.
    $node_1 = $this->createNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_list_integer' => [
        '2',
      ],
    ]);
    $this->assignAccessPolicy($node_1, 'security');

    // Create a basic node.
    $node_2 = $this->createNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
      'field_list_integer' => [
        '0',
      ],
    ]);
    $this->assignAccessPolicy($node_2, 'security');

    $user = $this->createUser([
      'access content',
      'view security content',
    ]);
    $user->save();

    $access = $this->accessPolicyValidator->validate($node_1, $user, 'view');
    $this->assertTrue($access, 'The user should have access');

    $access = $this->accessPolicyValidator->validate($node_2, $user, 'view');
    $this->assertFalse($access, 'The user should not have access');

    $this->setCurrentUser($user);
    $this->assertQueryResults('node', [
      $node_1->id(),
    ]);

  }

  /**
   * Tests comparing numeric list fields between the user and entity.
   */
  public function testFieldMatchListNumeric() {
    $policy = $this->entityTypeManager->getStorage('access_policy')->load('security');
    $access_rule = $this->accessRuleManager->getHandler('node', 'numeric_match_field_list_integer');
    $access_rule->setSettings([
      'operator' => '<=',
      'value' => ['field' => 'field_list_integer'],
    ]);
    $policy->addAccessRule($access_rule);
    $policy->save();

    $node_1 = $this->createNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_list_integer' => [
        '3',
      ],
    ]);
    $this->assignAccessPolicy($node_1, 'security');

    $node_2 = $this->createNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_list_integer' => [
        '2',
      ],
    ]);
    $this->assignAccessPolicy($node_2, 'security');

    $node_3 = $this->createNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
      'field_list_integer' => [
        '0',
      ],
    ]);
    $this->assignAccessPolicy($node_3, 'security');

    $user = $this->createUser([
      'access content',
      'view security content',
    ]);
    $user->set('field_list_integer', '2');
    $user->save();

    $access = $this->accessPolicyValidator->validate($node_1, $user, 'view');
    $this->assertFalse($access, 'The user should not have access');

    $access = $this->accessPolicyValidator->validate($node_2, $user, 'view');
    $this->assertTrue($access, 'The user should have access');

    $access = $this->accessPolicyValidator->validate($node_3, $user, 'view');
    $this->assertTrue($access, 'The user should have access');

    $this->setCurrentUser($user);
    $this->assertQueryResults('node', [
      $node_2->id(),
      $node_3->id(),
    ]);

  }

}
